/*
 *	AfterCheck
 *
 *	The function is used to check whether the first suspended crossing basepairs in the motif sequence appears in the target sequence
 *	The function checks and then updates the matchedpFin1 and matchedpFin2 information
 *	The function traverse each crossing basepair, and try to map it with the base basepair unmatched in target
 *	Then the newly mapped basepair is added as matched
 *	This approach is fine because we choose the four indices by finding the closest indices to the crossing basepair
 *	If the mapped basepair can map to another crossing basepair, then the crossing basepair is also covered by the four indices
 *	But that means the four indices is not the closest indices to the original crossing basepair
 *	This contradiction means the basepair in target cannot map to another crossing basepair
*/
double AfterCheck(char *seq1, char *seq2, int seqLeng1, int seqLeng2, 
		int *pair1, int *pair2, int numPair1, int numPair2, 
		int *crossPair, int numCrossPair, MATCH matchPairs, int *matchedpFin1, int *matchedpFin2){
	int i, j, k, l, m, n;
	int sid1 = 0, sid2 = seqLeng1, sid3 = 0, sid4 = seqLeng2;		/* the four sequence indices covers a crossing basepair*/
	int pid1 = -1, pid2 = -1, pid3 = -1, pid4 = -1;				/* the four basepair indices that contribute to 'sids'*/
	int loc1 = -1, loc2 = -1, loc3 = -1, loc4 = -1;				/* the location of its nucleotide, the left one or the right one*/
	int sidCrossLeft, sidCrossRight;					/* the indices for the crossing basepair*/
	int sidMotifLeft, sidMotifRight;					/* the indices of the motif basepair's left index and right index*/
	int sidTargetLeft, sidTargetRight;					/* the indices of the target basepair's left and right indices*/ 
	int choosenPair;							/* indicates which basepair in target is choosen for matching*/
	int PosTagMotif, PosTagTarget;						/* the position tags for motif and target, 1-n pairing*/
	int *matchedTargetIndex = (int*) ckalloc(numPair2*sizeof(int));		/* the array indicating which basepair is already matched*/
	double TempScore, OptScore;						/* the temporary score and the optimum score*/
	for(j=0; j<numPair2; ++j){
		matchedTargetIndex[j] = 0;
	}
	/* for each matched basepair*/	
	for(m=0; m<matchPairs.numbasepair; ++m){
		/* k and l are the indices of the matched basepairs in the pair1 and pair2 array, respectively*/
		k = matchPairs.pair[2 * m];
		l = matchPairs.pair[2 * m + 1];
		/* mark that pair as already matched, not available*/
		matchedTargetInedx[l] = 1;
		/* copy the matched basepairs*/
		for(n=0; n<6; ++n){
			matchedpFin1[6 * m + n] = pair1[6 * k + n];
			matchedpFin2[6 * m + n] = pair2[6 * l + n];
		}
	}

	for(i=0; i<numCrossPair; ++i){					/* each crossing pair*/
		/* records the indices*/		
		sidCrossLeft = crossPair[6 * i];
		sidCrossRight = crossPair[6 * i + 3];
		for(j=0; j<matchPairs.numbasepair; ++j){		/* each pair in matchedpFin1, not the original pair1*/
			sidMotifLeft = matchedpFin1[6 * j];
			sidMotifRight = matchedpFin1[6 * j + 3];
			/* for the two indices in the motif pair, compare to the currently maintained four indices*/
			if(sidMotifLeft > sid1 && sidMotifLeft <= sidCrossLeft){
				/* update the maintained four indices if one of the motif indices satisfy some conditions*/
				sid1 = sidMotifLeft;
				/* records the basepair index, and if it is left index or right index*/
				pid1 = j;
				loc1 = 0;				/* 0 indicates left while 1 indicates right*/
			}
			if(sidMotifLeft > sid3 && sidMotifLeft <= sidCrossRight){
				sid3 = sidMotifLeft;
				pid3 = j;
				loc3 = 0;
			}
			if(sidMotifRight > sid1 && sidMotifRight <= sidCrossLeft){
				sid1 = sidMotifRight;
				pid1 = j;
				loc1 = 1;
			}
			if(sidMotifRight > sid3 && sidMotifRight <= sidCrossRight){
				sid3 = sidMotifRight;
				pid3 = j;
				loc3 = 1;
			}
			if(sidMotifLeft < sid2 && sidMotifLeft >= sidCrossLeft){
				sid2 = sidMotifLeft;
				pid2 = j;
				loc2 = 0;			
			}
			if(sidMotifLeft < sid4 && sidMotifLeft >= sidCrossRight){
				sid4 = sidMotifLeft;
				pid4 = j;
				loc4 = 0;
			}
			if(sidMotifRight < sid2 && sidMotifRight >= sidCrossLeft){
				sid2 = sidMotifRight;
				pid2 = j;
				loc2 = 1;
			}
			if(sidMotifRight < sid4 && sidMotifRight >= sidCrossRight){
				sid4 = sidMotifRight;
				pid4 = j;
				loc4 = 1;
			}					
		}
		/* compute the postion tag value to avoid mismatching 1 - n cases*/
		PosTagMotif = 0;
		if(sidCrossLeft == sid1)	PosTagMotif += 1;
		if(sidCrossLeft == sid2)	PosTagMotif += 2;
		if(sidCrossLeft == sid3)	PosTagMotif += 4;
		if(sidCrossLeft == sid4)	PosTagMotif += 8;
		if(sidCrossRight == sid1)	PosTagMotif += 16;
		if(sidCrossRight == sid2)	PosTagMotif += 32;
		if(sidCrossRight == sid3)	PosTagMotif += 64;
		if(sidCrossRight == sid4)	PosTagMotif += 128;
		/* rebuild the four indices from the index from target*/
		sid1 = pair2[6 * pid1 + 3 * loc1];
		sid2 = pair2[6 * pid2 + 3 * loc2];
		sid3 = pair2[6 * pid3 + 3 * loc3];
		sid4 = pair2[6 * pid4 + 3 * loc4];
		/* if the four indices can not cover two regions then break*/
		if(!(sid1<= sid2 && sid2 <= sid3 && sid3 <= sid4)){
			continue;
		}
		/* initialize the optimum score and assume no pair matched*/
		OptScore = 0;
		choosenPair = -1;
		/* for each pair in pair2 (the original version)*/
		for(j=0; j<numPair2; ++j){
			/* if the basepair has already been matched*/
			if(matchedTargetIndex[j] == 1){
				continue;
			}
			sidTargetLeft = pair2[6 * j];
			sidTargetRight = pair2[6 * j + 3];
			/* check whether there is any pair in the region covered by the four indices*/
			if(sid1 <= sidTargetLeft && sidTargetLeft <= sid2 && sid3 <= sidTargetRight && sidTargetRight <= sid4){
				PosTagTarget = 0;
				if(sidTargetLeft == sid1)	PosTagTarget += 1;
				if(sidTargetLeft == sid2)	PosTagTarget += 2;
				if(sidTargetLeft == sid3)	PosTagTarget += 4;
				if(sidTargetLeft == sid4)	PosTagTarget += 8;
				if(sidTargetRight == sid1)	PosTagTarget += 16;
				if(sidTargetRight == sid2)	PosTagTarget += 32;
				if(sidTargetRight == sid3)	PosTagTarget += 64;
				if(sidTargetRight == sid4)	PosTagTarget += 128;
				/* if the two tags are not equal, then the 1-n pairing pattern between the two basepairs are not the same
				   which should be avoided*/
				if(PosTagMotif != PosTagTarget){
					continue;
				}
				/* computes the score*/
				TempScore = weight3 * RetrievePairScore(seq1, seq2, &crossPair[6 * i], &pair2[6 * j]);
				/* update to the best score*/
				if(TempScore > OptScore){
					OptScore = TempScore;
					choosenPair = j;
				}
			}
		}
		/* if some basepair has been matched*/
		if(choosenPair > 0){
			/* mark the pair as not available*/
			matchedTargetIndex[choosenPair] = 1;
			/* add it to matchPairs.score*/
			matchPairs.score += OptScore;
			/* add PENMissTarget to matchPairs.score*/
			matchPairs.score += PENMissTarget;
			/* increase matchPairs.numbasepair*/
			++matchPairs.numbasepair;
			/* add bases to matchedpFin1 and matchedpFin2*/
			for(n=0; n<6; ++n){
				matchedpFin1[6 * (numbasepair - 1) + n] = crossPair[6 * i + n];
				matchedpFin2[6 * (numbasepair - 1) + n] = pair2[6 * choosenPair + n];
			}
		}
	}
	return (matchPairs.score);
}

