#include "./inc/ComputeScore.h"
#include "./inc/ALIGN.h"
#include "./inc/RetrieveScore.h"
#include "./inc/ReadInput.h"
#include "./inc/Parameter.h"
#include <stdio.h>
#include <stdlib.h>

double max(int a, int b){
	return (double) ((a>b)?a:b);
}

/*
 * ComputeSeq
 *
 * The function is used to calculate the sequence similarity between two strings
 * The function calls 'ALIGN', which is defined in 'llmax.h' and 'llmax.c'
 * The input is the first sequence in NUMERIC format, the length of the first sequence to be compared,
 * the second sequence in NUMERIC format, and the length of the second sequence to be compared
 * REMEMBER THE STRING HAS BEEN CONVERTED INTO NUMERALS, ALTHOUGH IT IS STILL 'CHAR' TYPE 
 * Finally the score is returned, data type is converted to 'double'
*/
double ComputeSeq(char *seq1, int len1, char *seq2, int len2){
	int	i, j;
	int	*sapp;
	int	score;
	if(len1 <= 0 && len2 <= 0)	{
		return(0);
	} else if(len1 <= 0)	{
		return(-(rGAPST() + rGAPET() * len2 + rPENStacking()));
	} else if(len2 <= 0)	{
		return(-(rGAPST() + rGAPET() * len1 + rPENStacking()));
	}
	/* prepare input for calling function*/
	sapp =  (int*) calloc(2 * (len1 + len2 + 1), sizeof(int));
	/* calls the function ALIGN*/
	score = ALIGN(&seq1[-1], &seq2[-1], len1, len2, W, rGAPST(), rGAPET(), sapp);
	free((void *) sapp);
	return((double) score);
}

/*
 * FillHairpinSim
 *
 * The function computes the hairpin loop sequence similarity between any two base-pairs
 * and fills the result in the matrix 'HairpinSim', which is defined in 'ComputeScore.h'
*/
void FillHairpinSim(char *seq1, char *seq2, int *pair1, int numPair1, int *pair2,  int numPair2, 
			int numCleavage, int *Cleavages){
	extern double **HairpinSim;					// the variable is defined in 'ComputeScore.h'
	int i, j, k, l;
	int l1, l2;							// length of the hairpin region
	/* allocate memory here*/
	HairpinSim = (double **) calloc(numPair1, sizeof(double*));
	for(i = 0; i < numPair1; ++ i){
		HairpinSim[i] = (double *) calloc(numPair2, sizeof(double));
	}
	/* call fucntion 'ComputeSeq' and fill up the HairpinSim matrix*/
	//printf("Hairpin Similarity:\n");
	for(i = 0; i < numPair1; ++ i){
		l1 = pair1[6 * i + 3] - pair1[6 * i] - 1;
		for(j = 0; j < numPair2; ++ j){
			l2 = pair2[6 * j + 3] - pair2[6 * j] - 1;
			HairpinSim[i][j] = ComputeSeq(&seq1[pair1[6 * i] + 1], l1, &seq2[pair2[6 * j] + 1], l2);
			//printf("%f	", HairpinSim[i][j]);
		}
		//printf("\n");
	}
	return;
}

double GetHairpinSim(int i, int j){
	return (HairpinSim[i][j]);
}

/*
 * FillBasepairSim
 *
 * The function computes the base-pair similarity between any two base-pairs
 * and fill the results in the matrix 'BasepairSim', which is defined in 'ComputeScore.c'
*/
void FillBasepairSim(char *seq1, char *seq2, int *pair1, int numPairs1, int *pair2, int numPairs2){
	extern double **BasepairSim;					// the variable is defined in 'ComputeScore.h'
	extern int W[15][15];						// the weight function, defined in 'weight.h'
	int i, j;
	int l1, l2;
	double score1, score2;
	/* allocate memory*/
	BasepairSim = (double **) calloc(numPairs1, sizeof(double*));
	for(i = 0; i < numPairs1; ++ i){
		BasepairSim[i] = (double *) calloc(numPairs2, sizeof(double));
	}
	/* compute the scores, since only one nucleotide in seqeunce, direct compare*/
	//printf("Basepair similarity:\n");
	for(i = 0; i < numPairs1; ++ i){
		for(j = 0; j < numPairs2; ++ j){
			score1 = W[seq1[pair1[6 * i]]][seq2[pair2[6 * j]]];
			score2 = W[seq1[pair1[6 * i + 3]]][seq2[pair2[6 * j + 3]]];
			BasepairSim[i][j] = score1 + score2;
			//printf("%f	", BasepairSim[i][j]);
		}
		//printf("\n");
	}
	return;
}

double GetBasepairSim(int i, int j){
	return (BasepairSim[i][j]);
}
/*
 * AddParaScore
 *
 * This function is adding elements of a PARASCORE type into a linklist's head, and returning the new head address
 * The input list is the former head of the linklist, the last index of basepair 1, and the last index of basepair2, and the score of matching
 * the sequences between the last indices above and some basepair behind's first index
 * The new head address is returned
 * The function is called by 'ComputeParaScore'
*/
PARASCORE *AddParaScore(PARASCORE *parascore, int lastlist1, int lastlist2, double score2seq){
	/* allocate a new block for the socre*/
        PARASCORE *para;
        para = (PARASCORE *) calloc(1, sizeof(PARASCORE));
        para -> index1 = lastlist1;
        para -> index2 = lastlist2;
        para -> score = score2seq;
	/* adds the newly allocated block to the first node in the linked list*/
        para -> next = parascore;
        return(para);
}

/*
 * ComputeParaScore
 *
 * The function computes the ParaScore list given a pair of basepairs in each sequence
 * The result is to be filled into global variable: 'ParaScore'
 * The input list is the head address of the linklist, basepair1 index, basepair2 index, sequence 1, sequence 2, length of sequence1,
 * length of sequence 2, basepairs in sequence 1, basepairs in sequence 2, number of basepairs in sequence 1, number of basepairs in sequence 2
 * The address of the linklist head is returned
*/
PARASCORE* ComputeParaScore(PARASCORE *parascore, int mout1, int mout2, char *seq_b1, char *seq_b2, int len_seq1,
        			int len_seq2, int *pairs1, int num_pairs1, int *pairs2, int num_pairs2, 
				int numCleavage, int *Cleavages){
	int i, j, k, l, m, n, k1, k2, c;
        int l1, l2, l3, l4;
	/* the table to be filled by dynamic programming: score, deletion socre and insertion score*/
        int **score, *delscore, insscore;
        double dscore;
        int end1, end2;
	/* allocate memory for the score matrices*/
        score = (int **) calloc((1 + len_seq1), sizeof(int *));
        delscore = (int *) calloc((1 + len_seq2), sizeof(int));
        for(i = 0; i <= len_seq1; i ++) {
                score[i] = (int *) calloc((1 + len_seq2), sizeof(int));
        }
	/* gets the starting position of the backward dynamic programming, even though it is called end*/
        end1 = pairs1[6 * mout1];
        end2 = pairs2[6 * mout2];
	/* Fill in the alignment table ending at (mout1, mout2), i.e.
        pairs1[4 * mout1] - 1, pairs2[4 * mout2] + 1, using
        backward DP */

        /*initialize to gap start*/
        delscore[end2] = c = (double) -rGAPST();
	/* boundary cases*/
        score[end1][end2] = 0;
        for(j = end2 - 1; j > 0; j --)  {
                c = delscore[j] = score[end1][j] = c - (double) rGAPET();
        }
        c = (double) -rGAPST();
        for(i = end1 - 1; i >= 0; i --) {
		/* boundary cases 2*/
                c = insscore = score[i][end2] = c - (double) rGAPET();
                for(j = end2 - 1; j >= 0; j --) {
			/* compute insertion and deletion score to apply affine gap penalty*/
                        insscore = max(insscore, score[i][j + 1] - (double) rGAPST()) - (double) rGAPET();
                        delscore[j] = max(delscore[j], score[i + 1][j] - (double) rGAPST()) - (double) rGAPET();
                        /*backward dynamic programming, W is the array storing the scoring of sequence similarity alignment*/
                        score[i][j] = max(insscore, score[i + 1][j + 1] + (double) W[seq_b1[i]][seq_b2[j]]);
                        score[i][j] = max(delscore[j], score[i][j]);
			score[i][j] = (double) score[i][j];
                }
        }
	/* retrive the scores according to positions of base-pairs that preceeds mout1 and mout2*/
        for(i = 0; i < mout1; i ++)     {
                if((k1 = pairs1[6 * i + 3]) > end1)       continue;
                for(j = 0; j < mout2; j ++)     {
                        if((k2 = pairs2[6 * j + 3]) > end2)       continue;
 			if(k1 < end1 && k2 < end2)      {
				/* modified comparing original program*/
				/* original: dscore = score[k1][k2];*/
				/* modify for the reason that the nucleotides forming basepairs should be excluded*/
                                dscore = score[k1+1][k2+1];
                        } else if(k1 == end1 && k2 == end2)     {
				/* corresponding to 1-n pairing cases*/
				/* which is a good sign of matching, should not give negative value, value to be determined*/
                                dscore = 0;
                        } else if(k1 == end1)   {
                                dscore = -rGAPST() - rGAPET() * (end2 - k2 - 1) - rPENStacking();
                        } else if(k2 == end2)   {
                                dscore = -rGAPST() - rGAPET() * (end1 - k1 - 1) - rPENStacking();
                        }
			if(dscore < 0 && k1 < end1 && k2 < end2){
				for(k=0; k<numCleavage; ++k){
					if(k1 < Cleavages[k] && end1 >= Cleavages[k]){
						dscore = 0;
						break;
					}
				}
			}
                        /*assign dscore to proper location of parascore*/
                        parascore = AddParaScore(parascore, i, j, dscore);
                }
        }
	/* collect memory*/
        for(i = 0; i <= len_seq1; i ++) {
                free((void *) score[i]);
        }
        free((void **) score);
        free((void *) delscore);
        return(parascore);
}

/*
 * FillParaScore
 *
 * The function is an interface of calling 'ComputeParaScore' several times
 * It is used to fill the 'ParaScore' matrix
 * The input list contains: sequence 1, length of sequence 1, sequence 2, length of sequence 2, basepairs in sequence 1, basepairs in sequence 2,
 * number of basepairs in sequence 1, number of basepairs in sequence 2 
 * No value is returned
*/
void FillParaScore(char *seq1, int len_seq1, char *seq2, int len_seq2, int *pairs1, int numPairs1, int *pairs2, int numPairs2,
			int numCleavage, int *Cleavages){
	extern struct PARASCORE ***ParaScore;				// the variable is defined in 'ComputeScore.h'
	int i, j, k, l;
	/* allocate for the static array*/
	ParaScore = (PARASCORE ***) calloc(numPairs1, sizeof(PARASCORE **));
	for(i=0; i<numPairs1;  ++i){
		 ParaScore[i] = (PARASCORE **) calloc(numPairs2, sizeof(PARASCORE *));
	}
	/* sets the current pointer of parascore to NULL*/
	for(i=0; i<numPairs1; ++i){
		for(j=0; j<numPairs2; ++j){
			ParaScore[i][j] = NULL;
		}
	}
	/* fill the ParaScore matrix, each cell is actually a linked list*/
	for(i=0; i<numPairs1; ++i){
		for(j=0; j<numPairs2; ++j){
			ParaScore[i][j] = ComputeParaScore(ParaScore[i][j], i, j, seq1, seq2, len_seq1, len_seq2, pairs1, numPairs1, pairs2, numPairs2,
						numCleavage, Cleavages);
			
		}
	}
	return;
}

/*
 * GetParaScore
 *
 * The function is used to retierve parascore
 * The input list is the head of linklist, and two base-pairs' indices
 * The score is returned 
*/
double GetParaScore(PARASCORE *parascore, int index1, int index2){
        PARASCORE *para;

        para = parascore;
	/* following the linklist, until found the matched score*/
        while(para){
                if(para -> index1 == index1 && para -> index2 == index2){
                        return(para -> score);
                }
                para = para -> next;
        }
        return(-1);
}

PARASCORE *GetParaHead(int i, int j){
	return (ParaScore[i][j]);
}

void ListAllParaScore(PARASCORE *parascore){
	PARASCORE *para;

        para = parascore;
	/* following the linklist, until found the matched score*/
        while(para != NULL){
                para = para -> next;
        }
        return;
}

/*
 * ComputeLeftRightLoopSim
 *
 * The function is used to compute the two loop regions that are defined by two base-pairs with enclosing relationship
 * The left region score is stored in LeftLoopSim, which is defined in 'ComputeScore.h'
 * The right region score is stored  in RightLoopSim, which is defined in 'ComputeScore.h'
 * Note that the function is not the same as previous functions(FillHairpinSim, FillBasepairSim, FillParaScore),
 * Those functions fills the matrices that are globally defined, and globally used.
 * However, the Left/RightLoopSim is globally defined, by not globally used, as it is meaningful only given the enclosing base-pair
 * The Left/RightLoopSim matrices keep being modified/overwritten, but not newly defined each time.
 * Be called in function 'CalChianScore'
 * The input list is sequence 1, sequence 2, length of sequence 1, length of sequence 2, basepairs 1, number of basepairs in 1,
 * basepair2, number of basepairs in 2, the enclosing basepair1, the enclosing basepair2, downarray1, size of downarray1
 * downarray2, and size of downarray2
*/
void ComputeLeftRightLoopSim(char *seq_b1, char *seq_b2, int len_seq1, int len_seq2, 
			int *pair1, int numPair1, int *pair2, int numPair2, int mout1, int mout2,
		    int *downarray1, int size_downarray1, int *downarray2, int size_downarray2, int numCleavage, int *Cleavages, int LastTag){
	/*
	printf("###########\n");
	printf("hellow world\n");
	//printf("lasttag:	%d\n", LastTag);
	printf("mout:	%d	%d\n", mout1, mout2);
	printf("numpair:	%d	%d\n", numPair1, numPair2);
	*/
	extern double** LeftLoopSim;
	extern double** RightLoopSim;
	int i, j, k, l, p, q, m, n, c;
	int l1, l2, l3, l4;
	int **score1, **score2, *delscore, insscore;
	int start1, start2, end1, end2;

	score1 = (int **) calloc((1 + len_seq1), sizeof(int *));
	score2 = (int **) calloc((1 + len_seq1), sizeof(int *));
	delscore = (int *) calloc((1 + len_seq2), sizeof(int));
	for(i = 0; i <= len_seq1; i ++)	{
		score1[i] = (int *) calloc((1 + len_seq2), sizeof(int));
		score2[i] = (int *) calloc((1 + len_seq2), sizeof(int));
	}
	/*
	printf("first dimension of score: %d\n", 1 + len_seq1);
	printf("second dimension of score: %d\n", 1 + len_seq2);
	*/
	/* sets the range of alignment*/
	if(mout1 < numPair1 && mout2 < numPair2){
		start1 = pair1[6 * mout1];
		start2 = pair2[6 * mout2];
		end1 = pair1[6 * mout1 + 3];
		end2 = pair2[6 * mout2 + 3];
	}
	
	//printf("start1: %d	start2: %d	end1: %d	end2: %d\n", start1, start2, end1, end2);
	
	/* in the case of not last call*/
	if(LastTag == 1){
		
		//printf("what i am modifying\n");
		
		start1 = -1;
		start2 = -1;
		end1 = len_seq1;
		end2 = len_seq2;
	}
	/* Fill in the alignment table starting from (mout1, mout2), i.e.
	pair1[4 * mout1 + 1] + 1, pair2[4 * mout2 + 1] + 1	
	forward DP */
	
	delscore[0] = c = -rGAPST();
	score1[0][0] = 0;
	for(j = 1; j <= end2 - start1 - 1; j ++)	{
		c = delscore[j] = score1[0][j] = c - rGAPET();
	}
	c = -rGAPST();
	for(i = 1; i <= end1 - start2 - 1; i ++)	{
		c = insscore = score1[i][0] = c - rGAPET();
		for(j = 1; j <= end2 - start1 - 1; j ++)	{
			insscore = max(insscore, score1[i][j - 1] - rGAPST()) - rGAPET();
			delscore[j] = max(delscore[j], score1[i - 1][j] - rGAPST()) - rGAPET();
			/* Note the forwarding matrix is offset by 1*/
			score1[i][j] = max(insscore, score1[i - 1][j - 1] + W[seq_b1[i + start1 - 1]][seq_b2[j + start2 - 1]]);
			score1[i][j] = max(delscore[j], score1[i][j]);
		}
	}
	/*
	printf("score1:\n");
	for(i = 0; i < 1 + len_seq1; ++ i){
		for(j = 0 ; j < 1 + len_seq2; ++ j){
			printf("%d	", score1[i][j]);
		}
		printf("\n");
	}
	printf("\n");
	printf("break between forward DP and backward DP\n");
	*/
	/* Fill in the alignment table ending at (mout1, mout2), i.e.
	pair1[4 * mout1 + 2] - 1, pair2[4 * mout2 + 1] + 1, using
	backfard DP */
	
	delscore[0] = c = -rGAPST();
	score2[0][0] = 0;
	for(j = 1; j <= end2 - start1 - 1; j ++)	{
		c = delscore[j] = score2[0][j] = c - rGAPET();
	}
	c = -rGAPST();
	for(i = 1; i <= end1 - start2 - 1; i ++)	{
		c = insscore = score2[i][0] = c - rGAPET();
		for(j = 1; j <= end2 - start1 - 1; j ++)	{
			insscore = max(insscore, score2[i][j - 1] - rGAPST()) - rGAPET();
			delscore[j] = max(delscore[j], score2[i - 1][j] - rGAPST()) - rGAPET();
			/* Note the forwarding matrix is offset by 1*/
			score2[i][j] = max(insscore, score2[i - 1][j - 1] + W[seq_b1[end1 - i]][seq_b2[end2 - j]]);
			score2[i][j] = max(delscore[j], score2[i][j]);
		}
	}
	/*
	printf("score2:\n");
	for(i = 0; i < 1 + len_seq1; ++ i){
		for(j = 0 ; j < 1 + len_seq2; ++ j){
			printf("%d	", score2[i][j]);
		}
		printf("\n");
	}
	printf("\n");
	*/
	//printf("crash after the dynamic programming part\n");
	
	//printf("after alignment, before copying scores\n");
	/* Copy the loop score from the above tables*/
	for(i = 0; i < size_downarray1; i ++)	{
		for(j = 0; j < size_downarray2; j ++)	{
			/*
			printf("i: %d	j: %d\n", i, j);
			printf("sizedownarry1: %d	sizedownarray2: %d\n", size_downarray1, size_downarray2);
			printf("downpair1: %d	downpair2: %d\n", downarray1[i], downarray2[j]);
			printf("numPair1: %d	numPair2: %d\n", numPair1, numPair2);
			printf("mout1: %d	mout2: %d\n", mout1, mout2);
			printf("leftloopsim: %f\n", (double) score1[pair1[6 * downarray1[i]] + 1][pair2[6 * downarray2[j]] + 1]);
			printf("rightloopsim: %f\n", (double) score2[pair1[6 * downarray1[i] + 3] + 1][pair2[6 * downarray2[j] + 3] + 1]);
			*/
			/* Note the forwarding matrix is offset by 1*/
			//printf("DownArray index:	%d	%d\n", pair1[6 * downarray1[i]], pair2[6 * downarray2[j]]);
			int scoreIndex1 = pair1[6 * downarray1[i]] - start1 - 1;
			scoreIndex1 = scoreIndex1 > 0 ? scoreIndex1 : 0;
			int scoreIndex2 = pair2[6 * downarray2[j]] - start2 - 1;
			scoreIndex2 = scoreIndex2 > 0 ? scoreIndex2 : 0;
			LeftLoopSim[i][j] = (double) score1[scoreIndex1][scoreIndex2];
			if(pair1[6 * downarray1[i]] - start1 <= 1 && pair2[6 * downarray2[j]] - start2 > 1 ||
			   pair1[6 * downarray1[i]] - start1 > 1 && pair2[6 * downarray2[j]] - start2 <= 1)	{
				LeftLoopSim[i][j] -= rPENStacking();
			}
			//printf("After copying left loop sim\n");
			/*
			if(LeftLoopSim[i][j] < 0){
				for(l=0; l<numCleavage; ++l){
					if(pair1[6 * downarray1[i]] < Cleavages[l] && pair1[6 * downarray1[i] + 3] >= Cleavages[l]){
						LeftLoopSim[i][j] = 0.0;
					}
				}
			}
			*/
			scoreIndex1 = end1 - pair1[6 * downarray1[i] + 3] - 1;
			scoreIndex1 = scoreIndex1 > 0 ? scoreIndex1 : 0;
			scoreIndex2 = end2 - pair2[6 * downarray2[j] + 3] - 1;
			scoreIndex2 = scoreIndex2 > 0 ? scoreIndex2 : 0;
			RightLoopSim[i][j] = (double) score2[scoreIndex1][scoreIndex2];
			if(end1 - pair1[6 * downarray1[i] + 3] <= 1 && end2 - pair2[6 * downarray2[j] + 3] > 1 ||
			   end1 - pair1[6 * downarray1[i] + 3] > 1 && end2 - pair2[6 * downarray2[j] + 3] <= 1)	{
				RightLoopSim[i][j] -= rPENStacking();
			}
			//printf("LeftScore:	%f, RightScore:	%f\n", LeftLoopSim[i][j], RightLoopSim[i][j]);
			/*
			if(RightLoopSim[i][j] < 0){
				for(l=0; l<numCleavage; ++l){
					if(pair1[6 * downarray1[i]] < Cleavages[l] && pair1[6 * downarray1[i] + 3] >= Cleavages[l]){
						RightLoopSim[i][j] = 0.0;
					}
				}
			}
			*/
			//printf("%f	", LeftLoopSim[i][j]);
		}
		//printf("\n");
	}
	/*
	printf("crash after copying\n");
	*/
	for(i = 0; i <= len_seq1; i ++)	{
		free((void *) score1[i]);
		free((void *) score2[i]);
	}
	free((void **) score1);
	free((void **) score2);
	free((void *) delscore);
	//printf("before return\n");
	return;
}

int IsSameGGroup(int bp1Start, int bp1End, int bp1OutStart, int bp1OutEnd, int bp2Start, int bp2End, int bp2OutStart, int bp2OutEnd){
	if(bp1Start > bp1OutStart && bp1End < bp1OutEnd && bp2Start > bp2OutStart && bp2End < bp2OutEnd ||
	   bp1Start == bp1OutStart && bp1End < bp1OutEnd && bp2Start == bp2OutStart && bp2End < bp2OutEnd ||
	   bp1Start > bp1OutStart && bp1End == bp1OutEnd && bp2Start > bp2OutStart && bp2End == bp2OutEnd){
		return 1;
	}else{
		return 0;
	}
}

int CheckGGroup(int start, int end, int outStart, int outEnd){
	if(start > outStart && end < outEnd){
		return 1;
	}else if(start == outStart && end < outEnd){
		return 2;
	}else if(start > outStart && end == outEnd){
		return 3;
	}else{
		printf("error occur when determining G group!\n");
		printf("the four indices:start	end	outStart	outEnd\n");
		printf("%d	%d	%d	%d\n", start, end, outStart, outEnd);
		exit(0);
	}
}

int IsSameHGroup(int bp1Start, int bp1End, int bp1BehindStart, int bp1BehindEnd, int bp2Start, int bp2End, int bp2BehindStart, int bp2BehindEnd){
	if(bp1End < bp1BehindStart && bp2End < bp2BehindStart ||
	   bp1End == bp1BehindStart && bp2End == bp2BehindStart){
		return 1;
	}else{
		return 0;
	}
}

int CheckHGroup(int start, int end, int behindStart, int behindEnd){
	if(end < behindStart){
		return 1;
	}else if(end == behindStart){
		return 2;
	}else{
		printf("error occur when determining H group!\n");
		exit(0);
	}
}
/*
 *	CalChainScore
 *
 *	Given two basepairs, the function calcualtes the optimum score of matching the basepairs enclosed by the two given pairs
*/
double CalChainScore(
			/* index of the two stacks*/
			int mout1, 			/* index of the stack in the first sequence that enclosing the chain*/
			int mout2, 			/* index of the stack in the second sequence that enclosing the chain*/
			/* information of the first pair*/
			int *pair1, 		/* predicted basepairs*/
			int *downarray1, 	/* the basepairs that are enclosed by mout1*/
			int sizedownarray1, 	/* number of basepairs enclosed by mout1*/
			int numPair1,		/* number of predicted basepairs in sequence 1*/
			/* information of the second pair*/
			/* commets refers to the comments of the first pair*/
			int *pair2, 
			int *downarray2, 
			int sizedownarray2, 
			int numPair2,
			/* match is the matrix of matching base-pairs, corresponding to M[Pa, Pb] is the recursive function*/
			/* this is a global variable, passed in for dynamic programming*/
			MATCH **match, 
			int **numContStacking,
			/* matchchain is the score for doing dynamic programming of the chains, locally used*/
			MATCH *matchchain, 
			/* the score is the score of matching the sequence of the regions, assuming there is no basepair inside*/
			/* the score is compared inside the function, if it is higher, it is returned back*/
			double mscore0,
			/* the hairpin region inside the stack*/
			int range11, int range12, 
			int range21, int range22, 
			/* the sequences*/
			char *seq_b1, 
			char *seq_b2, 
			/* length of sequence*/
			int len_seq1, 
			int len_seq2,
			int numCleavage,
			int *Cleavages,
			int LastTag
){
	extern	double **LeftLoopSim, **RightLoopSim, **BasepairSim;
	extern	struct PARASCORE ***ParaScore;
	extern	int W[15][15]; 
	int	i, j, k, l, m1, m2, mk, ml, m, n, l1, l2;
	int	len1, len2;
	int	**lastlist1, *sizelist1, **lastlist2, *sizelist2;
	int	**ilast, **jlast;
	int	*numNestMotif, *numNestTarget, *numPreMotif, *numPreTarget, *numPostMotif, *numPostTarget;
	int	mini, minj, index2[2];
	double	**score, **multiscore, mscore, s;
	double	score2seq1 = 0.0, score2seq2 = 0.0;
	double	score2seq;
	double	isf = 0, s1, s2, s3, s4;
	int	**RightTag;
	/*
	double	tempPENMissMotif, tempPENMissTarget;
	tempPENMissMotif = rPENMissMotif();
	tempPENMissTarget = rPENMissTarget();
	*/
	/* boundary case*/
	if(sizedownarray1 == 0 || sizedownarray2 == 0)	{
		return(0);
	}
	/* change basepair missing penalty for the putative covering case*/
	/*
	if(range11 == range12 && range12 == range21 && range21 == range22 && range22 == -1){
		SetScore('u', 0.0);
		SetScore('v', 0.0);
	}
	*/
	/* the calculations of energy part are removed here, because we do not consider the energy contribution*/
	/* build last list here, the positional order of the base-pairs*/
	sizelist1 = (int *) calloc(sizedownarray1, sizeof(int));
	lastlist1 = (int **) calloc(sizedownarray1, sizeof(int *));
	for(i = 0; i < sizedownarray1; i ++)	{
		lastlist1[i] = (int *) calloc(sizedownarray1, sizeof(int));
	}
	sizelist2 = (int *) calloc(sizedownarray2, sizeof(int));
	lastlist2 = (int **) calloc(sizedownarray2, sizeof(int *));
	for(i = 0; i < sizedownarray2; i ++)	{
		lastlist2[i] = (int *) calloc(sizedownarray2, sizeof(int));
	}
	BuildLastlist(pair1, downarray1, sizedownarray1, lastlist1, sizelist1);
	BuildLastlist(pair2, downarray2, sizedownarray2, lastlist2, sizelist2);
	/*
	printf("printing lastlist1\n");
	for(i=0; i<sizedownarray1; ++i){
		for(j=0; j<sizelist1[i]; ++j){
			printf("%d	", lastlist1[i][j]);
		}
		printf("\n");
	}
	printf("printing lastlist2\n");
	for(i=0; i<sizedownarray2; ++i){
		for(j=0; j<sizelist2[i]; ++j){
			printf("%d	", lastlist2[i][j]);
		}
		printf("\n");
	}
	*/
	/* compute the loop similarity of regions that are sandwitched by two basepairs*/
	/* left side is stored in LeftLoopSim, right side is stored in RightLoopSim*/
	LeftLoopSim = (double **) calloc(numPair1, sizeof(double *));
	RightLoopSim = (double **) calloc(numPair1, sizeof(double *));
	RightTag = (int **) calloc(numPair1, sizeof(int *));
	for(i = 0; i < numPair1; i ++)	{
		LeftLoopSim[i] = (double *) calloc(numPair2, sizeof(double));
		RightLoopSim[i] = (double *) calloc(numPair2, sizeof(double));
		RightTag[i] = (int *) calloc(numPair2, sizeof(int));
	}
	/* initialzie RightTag to all 1, assuming all of them are taken*/
	for(i = 0; i < numPair1; ++ i){
		for(j = 0; j < numPair2; ++ j){
			RightTag[i][j] = 1;
		}
	}
	/* compute left loop similarity and right loop similarity*/
	ComputeLeftRightLoopSim(seq_b1, seq_b2, len_seq1, len_seq2, pair1, numPair1, pair2, numPair2,
			 mout1, mout2, downarray1, sizedownarray1, downarray2, sizedownarray2, numCleavage, Cleavages, LastTag);
	/*
	printf("Leftloop Similarity:\n");
	for(i = 0; i < numPair1; ++ i){
		for(j = 0; j < numPair2; ++j){
			printf("%f	", LeftLoopSim[i][j]);
		}
		printf("\n");
	}
	printf("Rightloop Similarity:\n");
	for(i = 0; i < numPair1; ++ i){
		for(j = 0; j < numPair2; ++j){
			printf("%f	", RightLoopSim[i][j]);
		}
		printf("\n");
	}
	printf("after computing LeftRightSim\n");
	*/
	/* sets the score of assuming no base-pair*/
	mscore = mscore0;
	/*
	printf("initial mscore: %f\n", mscore);
	*/
	mini = minj = -1;
	/* allocating memory for the matrices that stores the dynamic programming scores*/
	/* score :	the optimal score achieved assuming there are at least one basepair matched*/
	/* multiscore:	the score assuming there are multiple basepair matched*/
	/* ilast:	the matrix used to store the i-dimension index of the optimum score ever achieved*/
	/* jlast:	the matrix used to store the j-dimension index of the optimum score ever achieved*/
	score = (double **) calloc(sizedownarray1, sizeof(double *));
	multiscore = (double **) calloc(sizedownarray1, sizeof(double *));
	ilast = (int **) calloc(sizedownarray1, sizeof(int *));
	jlast = (int **) calloc(sizedownarray1, sizeof(int *));
	for(i = 0; i < sizedownarray1; i ++){
		score[i] = (double *) calloc(sizedownarray2, sizeof(double));
		multiscore[i] = (double *) calloc(sizedownarray2, sizeof(double));
		ilast[i] = (int *) calloc(sizedownarray2, sizeof(int));
		jlast[i] = (int *) calloc(sizedownarray2, sizeof(int));
	}
	/* computes the numNestMotif and numNestTarget, the number of basepair nested in m1 in motif and m2 in target*/
	numNestMotif = (int *) calloc(numPair1, sizeof(int));
	numNestTarget = (int *) calloc(numPair2, sizeof(int));
	numPreMotif = (int *) calloc(numPair1, sizeof(int));
	numPreTarget = (int *) calloc(numPair2, sizeof(int));
	numPostMotif = (int *) calloc(numPair1, sizeof(int));
	numPostTarget = (int *) calloc(numPair2, sizeof(int));
	for(i = 0; i < sizedownarray1; i ++){
		/* the basepair of interest is m1*/
		m1 = downarray1[i];
		numNestMotif[m1] = 0;
		for(k=0; k<sizedownarray1; ++k){
			mk = downarray1[k];
			if(pair1[6 * mk] >= pair1[6 * m1] && pair1[6 * mk + 3] <= pair1[6 * m1 + 3]){
				/* note that here includes the pair m1 itself*/				
				++numNestMotif[m1];
			}
			if(pair1[6 * mk] < pair1[6 * m1]){
				++numPreMotif[m1];
			}
			if(pair1[6 * mk] >= pair1[6 * m1] && pair1[6 * mk + 3] > pair1[6 * m1 + 3]){
				++numPostMotif[m1];	
			}
		}
	}
	/* checking sum of PreMotif, NestMotif and PostMotif*/
	for(i = 0; i<sizedownarray1; ++i){
		mk = downarray1[i];
		
		if(numPreMotif[mk] + numNestMotif[mk] + numPostMotif[mk] != sizedownarray1){
			printf("mout1: %d	mk: %d\n", mout1, mk);
			printf("PreMotif: %d	NestMotif: %d	PostMotif: %d	sizedownarray1: %d\n", numPreMotif[mk], numNestMotif[mk], numPostMotif[mk], sizedownarray1);
			printf("Check base-pair position Error, in Query!\n");
		}
	}
	for(i = 0; i < sizedownarray2; i ++){
		/* the basepair of interest is m1*/
		m2 = downarray2[i];
		numNestTarget[m2] = 0;
		for(k=0; k<sizedownarray2; ++k){
			mk = downarray2[k];
			if(pair2[6 * mk] >= pair2[6 * m2] && pair2[6 * mk + 3] <= pair2[6 * m2 + 3]){
				/* note that here includes the pair m1 itself*/	
				++numNestTarget[m2];
			}
			if(pair2[6 * mk] < pair2[6 * m2]){
				++numPreTarget[m2];
			}
			if(pair2[6 * mk] >= pair2[6 * m2] && pair2[6 * mk + 3] > pair2[6 * m2 + 3]){
				++numPostTarget[m2];	
			}
		}
	}
	/* checking sum of PreMotif, NestMotif and PostMotif*/
	for(i = 0; i<sizedownarray2; ++i){
		mk = downarray2[i];
		if(numPreTarget[mk] + numNestTarget[mk] + numPostTarget[mk] != sizedownarray2){
			printf("mout2: %d	mk: %d\n", mout2, mk);
			printf("PreTarget: %d	NestTarget: %d	PostTarget: %d	sizedownarray2: %d\n", numPreTarget[mk], numNestTarget[mk], numPostTarget[mk], sizedownarray2);
			printf("Check base-pair position Error, in Target!\n");
		}
	}
	int Stackingm1 = -1;
	int Stackingm2 = -1;
	double StackingSingleScore = -100.0;
	/*
	printf("printing downarray 1\n");
	for(i=0; i<sizedownarray1; ++i){
		printf("%d	", downarray1[i]);
	}
	printf("\n");
	printf("printing downarray 2\n");
	for(i=0; i<sizedownarray2; ++i){
		printf("%d	", downarray2[i]);
	}
	printf("\n");
	*/
	/* starting compute the score*/
	for(i = 0; i < sizedownarray1; i ++){
		for(j = 0; j < sizedownarray2; j ++){
			/*
			printf("mscore: %f\n", mscore);
			*/
			/* the orignal index of the two basepairs, the current checked basepair*/
			m1 = downarray1[i];
			m2 = downarray2[j];
			score[i][j] = 0.0;
			//printf("m1: %d, m2: %d\n", m1, m2);
			/* Compute the sequence similarity between two loop sequences*/
			/* leftloop and rightloop are indexed by downarray1 and downarray2, not the original indeces*/
			/* records the left/right loop similarity*/
			/*
			printf("mout1: %d	mout2: %d	m1: %d	m2: %d\n", mout1, mout2, m1, m2);
			printf("i: %d	j: %d\n", i, j);
			*/
			score2seq1 = LeftLoopSim[i][j];
			/*
			printf("LeftLoopSim[i][j]: %f\n", score2seq1);
			*/
			score2seq2 = RightLoopSim[i][j];
			/*
			printf("RightLoopSim[i][j]: %f\n", score2seq2);
			*/
			//printf("before check G group\n");
			/* check for whether m1 and mout1, m2 and mout2 belong to the same enclosing group*/
			if(LastTag == 1 ||
			   CheckGGroup(pair1[6 * m1], pair1[6 * m1 + 3], pair1[6 * mout1], pair1[6 * mout1 + 3]) ==
			   CheckGGroup(pair2[6 * m2], pair2[6 * m2 + 3], pair2[6 * mout2], pair2[6 * mout2 + 3])){
				//printf("success after checking G group\n");
				/* if the two basepairs share some common nucleotide*/
				if(CheckGGroup(pair1[6 * m1], pair1[6 * m1 + 3], pair1[6 * mout1], pair1[6 * mout1 + 3]) == 2){
					/* substract the sequence similarity*/
					score[i][j] -= rweight2() * (double) W[seq_b1[pair1[6 * m1]]][seq_b2[pair2[6 * m2]]];
					/* we can add bounus here*/
					score[i][j] += rweight3() * (double) rBonusSameG();
				}else if(CheckGGroup(pair1[6 * m1], pair1[6 * m1 + 3], pair1[6 * mout1], pair1[6 * mout1 + 3]) == 3){
					/* substract the sequence similarity*/
					score[i][j] -= rweight2() * (double) W[seq_b1[pair1[6 * m1 + 3]]][seq_b2[pair2[6 * m2 + 3]]];
					/* we can add bounus here*/
					score[i][j] += rweight3() * (double) rBonusSameG();
				}	
				//printf("after checking 1-n pairing\n");
				/* assuming there is only one basepair matched, and is pair i,j, calculate the hairpin sequence similarity*/
				/* the score of matching the two base-pairs only, remember to add the penalty of missing existing basepairs*/
				/*
				printf("initial score: %f\n", score[i][j]);
				printf("match[m1][m2]: %f\n", match[m1][m2].score);
				printf("score2seq1: %f\n", score2seq1);
				printf("score2seq2: %f\n", score2seq2);
				printf("sizedownarray1 : %d\n", sizedownarray1);
				printf("sizedownarray2 : %d\n", sizedownarray2);
				printf("numNestMotif m1: %d\n", numNestMotif[m1]);
				printf("numNestTarget m2: %d\n", numNestTarget[m2]);
				printf("rPENMissMotif: %f\n", rPENMissMotif());
				printf("rPENMissTarget: %f\n", rPENMissTarget());
				*/
				score[i][j] += match[m1][m2].score + rweight1() * (score2seq1 + score2seq2)
					- rweight3() * rPENMissMotif() * (sizedownarray1 - numNestMotif[m1]) 
					- rweight3() * rPENMissTarget() * (sizedownarray2 - numNestTarget[m2]);
				/*
				printf("single score before checking local alignment %f\n", score[i][j]);
				*/
				/* if cleavage sites presents in the region, we do not substract base-pair miss penalty*/
				/* the left side*/
				for(l = 0; l < numCleavage; ++l){
					if(pair1[6 * mout1] < Cleavages[l] && pair1[6 * m1] >= Cleavages[l] || LastTag == 1){
						/* check for penalty in the structure*/
						double PenaltyScore = numPreMotif[m1] * rweight3() * rPENMissMotif()
									+ numPreTarget[m2] * rweight3() * rPENMissTarget();
						/* in the case we shoule not consider the left part into our alignment
						   because the penalty is greater than the credit we get from mapping the sequence*/
						if(PenaltyScore > rweight1() * score2seq1){
							score[i][j] += PenaltyScore - rweight1() * score2seq1;
						}
						/* we only add back once, even with multiple cleavage sites presents*/
						break;
					}
				}
				/* the right side*/
				for(l = 0; l < numCleavage; ++l){
					if(pair1[6 * m1 + 3] < Cleavages[l] && pair1[6 * mout1 + 3] >= Cleavages[l] || LastTag == 1){
						/* check for penalty in the structure*/
						double PenaltyScore = numPostMotif[m1] * rweight3() * rPENMissMotif()
									+numPostTarget[m2] * rweight3() * rPENMissTarget();
						/* in the case we shoule not consider the right part into our alignment
						   because the penalty is greater than the credit we get from mapping the sequence*/
						if(PenaltyScore > rweight1() * score2seq2){
							score[i][j] += PenaltyScore - rweight1() * score2seq2;
							RightTag[m1][m2] = 0;
						}
						/* we only add back once, even with multiple cleavage sites presents*/
						break;
					}
				}
				/* in case there is no cleavage site*/
				if(numCleavage == 0 && LastTag == 1){
					/* the left side*/
					/* check for penalty in the structure*/
					double PenaltyScore = numPreMotif[m1] * rweight3() * rPENMissMotif()
								+ numPreTarget[m2] * rweight3() * rPENMissTarget();
					/* in the case we shoule not consider the left part into our alignment
					   because the penalty is greater than the credit we get from mapping the sequence*/
					if(PenaltyScore > rweight1() * score2seq1){
						score[i][j] += PenaltyScore - rweight1() * score2seq1;
					}
					/* the right side*/
					/* check for penalty in the structure*/
					PenaltyScore = numPostMotif[m1] * rweight3() * rPENMissMotif()
								+numPostTarget[m2] * rweight3() * rPENMissTarget();
					/* in the case we shoule not consider the right part into our alignment
					   because the penalty is greater than the credit we get from mapping the sequence*/
					if(PenaltyScore > rweight1() * score2seq2){
						score[i][j] += PenaltyScore - rweight1() * score2seq2;
						RightTag[m1][m2] = 0;
					}
				}
				/* add bonus to stacking effect*/
				if(pair1[6 * m1] == pair1[6 * mout1] + 1 && pair1[6 * m1 + 3] == pair1[6 * mout1 + 3] - 1 &&
				   pair2[6 * m2] == pair2[6 * mout2] + 1 && pair2[6 * m2 + 3] == pair2[6 * mout2 + 3] - 1 &&
				   range11 >= 0 && range12 >= 0 && range21 >= 0 && range22 >= 0){
					Stackingm1 = m1;
					Stackingm2 = m2;
					score[i][j] += rweight3() * rBonusStacking() * (numContStacking[m1][m2] + 1);
					//printf("added score: %f\n", rweight3() * rBonusStacking() * (numContStacking[m1][m2] + 1));
					
				}
				StackingSingleScore = score[i][j];
				/* initializing the score to the case that the i,j basepairs are the first basepair matched
				   which is equals to the single score calculated before*/
				multiscore[i][j] = score[i][j];
				
				
			}else{
				//printf("fail after checking G group\n");
				score[i][j] = - rweight3() * rHARDPENG();
				multiscore[i][j] = score[i][j];
			}
			/*
			printf("important! score before checking multiloop: %f\n", score[i][j]);
			*/
			//printf("finish single score\n");
			ilast[i][j] = jlast[i][j] = -1;
			/* look for the higher score around the edge of the matrix: score[i][x] and score[x][j]*/
			/* while x is variable and i, j are fixed*/
			/* update the score and record the index in ilast and jlast*/
			/* also remember to add the basepair missing penalty*/
			for(m = 0; m < sizelist1[i]; m ++){
				s = score[lastlist1[i][m]][j];
				if(s > score[i][j]){
					score[i][j] = s;
					ilast[i][j] = lastlist1[i][m];
					jlast[i][j] = j;
				}
			}
			for(n = 0; n < sizelist2[j]; n ++){
				s = score[i][lastlist2[j][n]];
				if(s > score[i][j]){
					score[i][j] = s;
					ilast[i][j] = i;
					jlast[i][j] = lastlist2[j][n];
				}
			}
			/*
			printf("score after checking previous matched pairs: %f\n", score);
			*/
			/* Compute the sequence similarity between two loop sequences*/
			/* The recomputation corresponds to the multi-base-pair case*/
			/* Re-compute the nested loop similarity each time*/
			/* The complexity to compute this is at most O(m * n), because
				the backtrack graph should be a tree -- there should be no
				cycle in the graph. The total number of backtracks that
				will be enuminated is at most the number of edges
				in the graph, i. e. m - 1. Therefore
				the overall computational complexity is at most O(m * n).*/
			for(m = 0; m < sizelist1[i]; m ++){
				for(n = 0; n < sizelist2[j]; n ++){
					/* retrieves the optimal score achieved previoursly in the score matrix*/
					/* diagnally, not restricted to the edges, because edges are checked previously*/
					/* puts the indices in index2*/	
					GetLastlist(lastlist1[i][m], lastlist2[j][n], ilast, jlast, index2);
					isf = 0.0;
					if(CheckHGroup(pair1[6 * downarray1[index2[0]]], pair1[6 * downarray1[index2[0]] + 3], 
							pair1[6 * m1], pair1[6 * m1 + 3]) ==
					   CheckHGroup(pair2[6 * downarray2[index2[1]]], pair2[6 * downarray2[index2[1]] + 3],
							pair2[6 * m2], pair2[6 * m2 + 3]) &&
					   (CheckGGroup(pair1[6 * m1], pair1[6 * m1 + 3], pair1[6 * mout1], pair1[6 * mout1 + 3]) ==
			   		    CheckGGroup(pair2[6 * m2], pair2[6 * m2 + 3], pair2[6 * mout2], pair2[6 * mout2 + 3]) ||
					    LastTag == 1)	
					){
						if(CheckHGroup(pair1[6 * downarray1[index2[0]]], pair1[6 * downarray1[index2[0]] + 3], 
							pair1[6 * m1], pair1[6 * m1 + 3]) == 2){
							//printf("added H group 2\n");
							/* substract the sequence score here*/
							isf -= rweight2() * (double) W[seq_b1[pair1[6 * m1]]][seq_b2[pair2[6 * m2]]];
							/* we can add bouns here*/
							isf += rweight3() * rBonusSameH();
							//printf("H Bonus added	m1: %d	m2: %d\n", m1, m2);
						}
						
						if(CheckGGroup(pair1[6 * m1], pair1[6 * m1 + 3], pair1[6 * mout1], pair1[6 * mout1 + 3]) == 3){
							/* substract the sequence similarity*/
							isf -= rweight2() * (double) W[seq_b1[pair1[6 * m1 + 3]]][seq_b2[pair2[6 * m2 + 3]]];
							/* we can add bounus here*/
							isf += rweight3() * (double) rBonusSameG();
						}
						
						/* dealing with the current basepair, sequence and basepair similarity*/
						isf += match[m1][m2].score;
						/*
						printf("m1: %d	m2: %d\n", m1, m2);
						printf("index2[0]: %d	index2[1]: %d\n", index2[0], index2[1]);
						printf("multiscore[index2[0]][index2[1]]: %f\n", multiscore[index2[0]][index2[1]]);
						printf("isf: %f\n", isf);
						*/
						/* gets the new hairpin loop score*/
						score2seq1 = GetParaScore(ParaScore[m1][m2], downarray1[index2[0]], downarray2[index2[1]]);
						/* add the right hairpin loop score and the missing basepair penalty (RightLoopSim)*/
						double PenaltyScore;
						/* if the previous pair has taken its right part into account, 
						   we need to reverse the effect, because afterwards when we add up the score, we have
						   taken the score corresponding to that part, which should not be taken*/
						if(RightTag[downarray1[index2[0]]][downarray2[index2[1]]] == 1){
							PenaltyScore = rweight3() * rPENMissMotif() * numPostMotif[downarray1[index2[0]]]
							 	    + rweight3() * rPENMissTarget() * numPostTarget[downarray2[index2[1]]];
							isf += PenaltyScore - rweight1() * RightLoopSim[index2[0]][index2[1]];
						}
						/* adds the basepair missing penalty*/
						/*
						printf("numPostMotif: %d\n", numPostMotif[downarray1[index2[0]]]);
						printf("numPostTarget: %d\n", numPostTarget[downarray2[index2[1]]]);
						printf("isf3: %f\n", isf);
						*/
						/* add the score of region after the final matched base-pair*/
						
						if(RightTag[m1][m2] == 1){
							isf += rweight1() * RightLoopSim[i][j];
							isf -= rweight3() * rPENMissMotif() * numPostMotif[downarray1[i]]
							 	    + rweight3() * rPENMissTarget() * numPostTarget[downarray2[j]];
						}
						
						/* add the new interval's sequence similarity and penalty (parascore)*/
						PenaltyScore = rweight3() * rPENMissMotif() 
								* (m1 - downarray1[index2[0]] - numNestMotif[downarray1[index2[0]]]) 
							 	    + rweight3() * rPENMissTarget() 
								* (m2 - downarray2[index2[1]] - numNestTarget[downarray2[index2[1]]]);
						/*						
						printf("Penaltyscore: %f\n", PenaltyScore);
						*/
						isf += rweight1() * score2seq1 - PenaltyScore;
						/* if we wish not to take the region into account*/
						if(rweight1() * score2seq1 < PenaltyScore){
							/* check whether there is a cleavage site*/
							for(l=0; l<numCleavage; ++l){
								if(pair1[6 * downarray1[index2[0]] + 3] < Cleavages[l] && pair1[6 * m1] >= Cleavages[l]){
									isf += PenaltyScore - rweight1() * score2seq1;
									break;
								}
							}
						}
						/*
						printf("isf7: %f\n", isf);
						*/						
					}else{
						isf = - rweight3() * rHARDPENH();
					}
					/* see if it is possible to beat the score of taking i,j as the start of the chain*/
					if((s = multiscore[index2[0]][index2[1]] + isf) > multiscore[i][j]){
						multiscore[i][j] = s;
						if(s > score[i][j]){
							score[i][j] = s;
							ilast[i][j] = index2[0];
							jlast[i][j] = index2[1];
						}
					}
				}
			}
			/* recording the score, and indices of matching basepairs*/
			if(score[i][j] > mscore)	{
				mscore = score[i][j];
				mini = i;
				minj = j;
			}
			/*
			printf("important! score after checking multiloop: %f\n", score[i][j]);
			*/
		}
	}
	matchchain -> numbasepair = 0;
	i = j = -1;
	while(mini >= 0 && minj >= 0)	{
		if(mini != i && minj != j)	{
			/* match->pair stores the matched basepairs in the two sequences*/
			/* adding the matched basepair in the first sequence*/
			matchchain -> pair[2 * matchchain -> numbasepair] = downarray1[mini];
			/* adding the matched basepair in the second sequence*/
			matchchain -> pair[2 * matchchain -> numbasepair + 1] = downarray2[minj];
			/* increase the number of matched basepair*/
			matchchain -> numbasepair ++;
		}
		i = mini;
		j = minj;
		mini = ilast[i][j];
		minj = jlast[i][j];
	}

	/* set back the Penalty for motif and target*/
	/*
	SetScore('u', tempPENMissMotif);
	SetScore('v', tempPENMissTarget);
	*/
	/*releasing memory*/
	for(i = 0; i < sizedownarray1; i ++)	{
		free((void *) score[i]);
		free((void *) multiscore[i]);
		free((void *) ilast[i]);
		free((void *) jlast[i]);
		free((void *) lastlist1[i]);
	}
	free((void **) multiscore);
	free((void **) score);
	free((void **) ilast);
	free((void **) jlast);
	free((void **) lastlist1);
	free((void *) sizelist1);
	for(i = 0; i < sizedownarray2; i ++)	{
		free((void *) lastlist2[i]);
	}
	free((void **) lastlist2);
	free((void *) sizelist2);
	for(i = 0; i < numPair1; i ++)	{
		free((void*) LeftLoopSim[i]);
		free((void*) RightLoopSim[i]);
		free((void*) RightTag[i]);
	}
	free((void **) LeftLoopSim);
	free((void **) RightLoopSim);
	free((void **) RightTag);
	free((void *) numNestMotif);
	free((void *) numNestTarget);
	free((void *) numPreMotif);
	free((void *) numPreTarget);
	free((void *) numPostMotif);
	free((void *) numPostTarget);
	return(mscore);
}

PARASCORE *FreeParaLoop(PARASCORE *parascore){
	PARASCORE *para;
	while(parascore){
		para = parascore->next;
		free((void *) parascore);
		parascore = para;
	}
	return(parascore);
}

void FreeLoopsim(int numPairs1, int numPairs2){
	extern	double **HairpinSim, **BasepairSim;
	extern	struct PARASCORE ***ParaScore;
	int	i, j;
	/* collects the memory allocated for score matrices*/
	for(i = 0; i < numPairs1; i ++)	{
		for(j = 0; j < numPairs2; j ++)	{
			ParaScore[i][j] = FreeParaLoop(ParaScore[i][j]);
		}
		free((void *) HairpinSim[i]);
		free((void *) BasepairSim[i]);
		free((void **) ParaScore[i]);
	}
	free((void **) HairpinSim);
	free((void **) BasepairSim);
	free((void ***) ParaScore);
	return;
}

/*
 *	AfterCheck
 *
 *	The function is used to check whether the first suspended crossing basepairs in the motif sequence appears in the target sequence
 *	The function checks and then updates the matchedpFin1 and matchedpFin2 information
 *	The function traverse each crossing basepair, and try to map it with the base basepair unmatched in target
 *	Then the newly mapped basepair is added as matched
 *	This approach is fine because we choose the four indices by finding the closest indices to the crossing basepair
 *	If the mapped basepair can map to another crossing basepair, then the crossing basepair is also covered by the four indices
 *	But that means the four indices is not the closest indices to the original crossing basepair
 *	This contradiction means the basepair in target cannot map to another crossing basepair
*/
double AfterCheck(char *seq1, char *seq2, int seqLeng1, int seqLeng2, 
		int *pair1, int *pair2, int numPair1, int numPair2, 
		int *crossPair, int numCrossPair, MATCH *matchPairs, int *matchedpFin1, int *matchedpFin2){
	int i, j, k, l, m, n;
	char *seq_b1, *seq_b2;
	int sid1 = 0, sid2 = seqLeng1, sid3 = 0, sid4 = seqLeng2;		/* the four sequence indices covers a crossing basepair*/
	int pid1 = -1, pid2 = -1, pid3 = -1, pid4 = -1;				/* the four basepair indices that contribute to 'sids'*/
	int loc1 = -1, loc2 = -1, loc3 = -1, loc4 = -1;				/* the location of its nucleotide, the left one or the right one*/
	int sidCrossLeft, sidCrossRight;					/* the indices for the crossing basepair*/
	int sidMotifLeft, sidMotifRight;					/* the indices of the motif basepair's left index and right index*/
	int sidTargetLeft, sidTargetRight;					/* the indices of the target basepair's left and right indices*/ 
	int choosenPair;							/* indicates which basepair in target is choosen for matching*/
	int PosTagMotif, PosTagTarget;						/* the position tags for motif and target, 1-n pairing*/
	int *matchedTargetIndex = (int*) calloc(numPair2, sizeof(int));		/* the array indicating which basepair is already matched*/
	double TempScore, OptScore;						/* the temporary score and the optimum score*/
	
	seq_b1 = (char*) calloc(MAXSEQ, sizeof(char));
	seq_b2 = (char*) calloc(MAXSEQ, sizeof(char));
	for(i = 0; i < seqLeng1; i ++)	{
		seq_b1[i] = Char2int(seq1[i]);
	}
	for(i = 0; i < seqLeng2; i ++)	{
		seq_b2[i] = Char2int(seq2[i]);
	}

	for(j=0; j<numPair2; ++j){
		matchedTargetIndex[j] = 0;
	}
	/* for each matched basepair*/	
	for(m=0; m<matchPairs->numbasepair; ++m){
		/* k and l are the indices of the matched basepairs in the pair1 and pair2 array, respectively*/
		k = matchPairs->pair[2 * m];
		l = matchPairs->pair[2 * m + 1];
		//printf("k: %d	l: %d\n", k, l);
		/* mark that pair as already matched, not available*/
		matchedTargetIndex[l] = 1;
		/* copy the matched basepairs*/
		for(n=0; n<6; ++n){
			matchedpFin1[6 * m + n] = pair1[6 * k + n];
			matchedpFin2[6 * m + n] = pair2[6 * l + n];
		}
		//printf("%d	%d	|	%d	%d\n", pair1[6 * k], pair1[6 * k + 3], pair2[6 * l], pair2[6 * l + 3]);
	}
	/* sort the matchedpFin array*/
	for(i=0; i<matchPairs->numbasepair; ++i){
		for(j=i+1; j<matchPairs->numbasepair; ++j){
			int temp;
			if(matchedpFin1[6 * i] > matchedpFin1[6 * j]){
				/* swap the two basepair and its corresponding part in matchedpFin2*/
				for(k=0;k<6;++k){
					/* swap basepair in sequence 1*/
					temp = matchedpFin1[6 * i + k];
					matchedpFin1[6 * i + k] = matchedpFin1[6 * j + k];
					matchedpFin1[6 * j + k] = temp;
					/* swap basepair in sequence 2*/
					temp = matchedpFin2[6 * i + k];
					matchedpFin2[6 * i + k] = matchedpFin2[6 * j + k];
					matchedpFin2[6 * j + k] = temp;
				}
			}
		}
	}
	for(i=0; i<numCrossPair; ++i){					/* each crossing pair*/
		/* records the indices*/		
		sidCrossLeft = crossPair[6 * i];
		sidCrossRight = crossPair[6 * i + 3];
		for(j=0; j<matchPairs->numbasepair; ++j){		/* each pair in matchedpFin1, not the original pair1*/
			sidMotifLeft = matchedpFin1[6 * j];
			sidMotifRight = matchedpFin1[6 * j + 3];
			/* for the two indices in the motif pair, compare to the currently maintained four indices*/
			if(sidMotifLeft > sid1 && sidMotifLeft <= sidCrossLeft){
				/* update the maintained four indices if one of the motif indices satisfy some conditions*/
				sid1 = sidMotifLeft;
				/* records the basepair index, and if it is left index or right index*/
				pid1 = j;
				loc1 = 0;				/* 0 indicates left while 1 indicates right*/
			}
			if(sidMotifLeft > sid3 && sidMotifLeft <= sidCrossRight){
				sid3 = sidMotifLeft;
				pid3 = j;
				loc3 = 0;
			}
			if(sidMotifRight > sid1 && sidMotifRight <= sidCrossLeft){
				sid1 = sidMotifRight;
				pid1 = j;
				loc1 = 1;
			}
			if(sidMotifRight > sid3 && sidMotifRight <= sidCrossRight){
				sid3 = sidMotifRight;
				pid3 = j;
				loc3 = 1;
			}
			if(sidMotifLeft < sid2 && sidMotifLeft >= sidCrossLeft){
				sid2 = sidMotifLeft;
				pid2 = j;
				loc2 = 0;			
			}
			if(sidMotifLeft < sid4 && sidMotifLeft >= sidCrossRight){
				sid4 = sidMotifLeft;
				pid4 = j;
				loc4 = 0;
			}
			if(sidMotifRight < sid2 && sidMotifRight >= sidCrossLeft){
				sid2 = sidMotifRight;
				pid2 = j;
				loc2 = 1;
			}
			if(sidMotifRight < sid4 && sidMotifRight >= sidCrossRight){
				sid4 = sidMotifRight;
				pid4 = j;
				loc4 = 1;
			}					
		}
		/* compute the postion tag value to avoid mismatching 1 - n cases*/
		PosTagMotif = 0;
		if(sidCrossLeft == sid1)	PosTagMotif += 1;
		if(sidCrossLeft == sid2)	PosTagMotif += 2;
		if(sidCrossLeft == sid3)	PosTagMotif += 4;
		if(sidCrossLeft == sid4)	PosTagMotif += 8;
		if(sidCrossRight == sid1)	PosTagMotif += 16;
		if(sidCrossRight == sid2)	PosTagMotif += 32;
		if(sidCrossRight == sid3)	PosTagMotif += 64;
		if(sidCrossRight == sid4)	PosTagMotif += 128;
		/* rebuild the four indices from the index from target*/
		sid1 = matchedpFin2[6 * pid1 + 3 * loc1];
		sid2 = matchedpFin2[6 * pid2 + 3 * loc2];
		sid3 = matchedpFin2[6 * pid3 + 3 * loc3];
		sid4 = matchedpFin2[6 * pid4 + 3 * loc4];
		/* if the four indices can not cover two regions then break*/
		if(!(sid1<= sid2 && sid2 <= sid3 && sid3 <= sid4)){
			/* substract the penalty in motif*/
			matchPairs->score -= rweight3() * rPENMissMotif();
			continue;
		}
		/* initialize the optimum score and assume no pair matched*/
		OptScore = 0;
		choosenPair = -1;
		/* for each pair in pair2 (the original version)*/
		for(j=0; j<numPair2; ++j){
			/* if the basepair has already been matched*/
			if(matchedTargetIndex[j] == 1){
				continue;
			}
			sidTargetLeft = pair2[6 * j];
			sidTargetRight = pair2[6 * j + 3];
			/* check whether there is any pair in the region covered by the four indices*/
			if(sid1 <= sidTargetLeft && sidTargetLeft <= sid2 && sid3 <= sidTargetRight && sidTargetRight <= sid4){
				PosTagTarget = 0;
				if(sidTargetLeft == sid1)	PosTagTarget += 1;
				if(sidTargetLeft == sid2)	PosTagTarget += 2;
				if(sidTargetLeft == sid3)	PosTagTarget += 4;
				if(sidTargetLeft == sid4)	PosTagTarget += 8;
				if(sidTargetRight == sid1)	PosTagTarget += 16;
				if(sidTargetRight == sid2)	PosTagTarget += 32;
				if(sidTargetRight == sid3)	PosTagTarget += 64;
				if(sidTargetRight == sid4)	PosTagTarget += 128;
				/* if the two tags are not equal, then the 1-n pairing pattern between the two basepairs are not the same
				   which should be avoided*/
				if(PosTagMotif != PosTagTarget){
					continue;
				}
				/* computes the score*/
				TempScore = rweight3() * RetrievePairScore(seq_b1, seq_b2, &crossPair[6 * i], &pair2[6 * j]);
				/*
				printf("TempScore1: %f\n", TempScore);
				*/
				/* calculate the sequence similarity*/
				double inbpSim = 0.0;
				if(sidTargetLeft != sid1 && sidTargetLeft != sid2){
					inbpSim += (double) W[seq_b1[crossPair[6 * i]]][seq_b2[pair2[6 * j]]];
				}else{
					TempScore += rweight3() * (rBonusSameG() + rBonusSameH()) / 2;
					/*
					printf("TempScore2: %f\n", TempScore);
					*/
				}
				if(sidTargetRight != sid3 && sidTargetRight != sid4){
					inbpSim += (double) W[seq_b1[crossPair[6 * i + 3]]][seq_b2[pair2[6 * j + 3]]];
				}else{
					TempScore += rweight3() * (rBonusSameG() + rBonusSameH()) / 2;
					/*
					printf("TempScore3: %f\n", TempScore);
					*/
				}
				TempScore += rweight2() * inbpSim;
				TempScore += rweight3() * rPENMissTarget();
				/*
				printf("TempScore4: %f\n", TempScore);
				*/
				/* update to the best score*/
				if(TempScore > OptScore){
					OptScore = TempScore;
					choosenPair = j;
				}
			}
		}
		/* if some basepair has been matched*/
		if(choosenPair > 0){
			/* mark the pair as not available*/
			matchedTargetIndex[choosenPair] = 1;
			/* add it to matchPairs.score*/
			matchPairs->score += OptScore;
			/* increase matchPairs.numbasepair*/
			++matchPairs->numbasepair;
			/* add bases to matchedpFin1 and matchedpFin2*/
			for(n=0; n<6; ++n){
				matchedpFin1[6 * (matchPairs->numbasepair - 1) + n] = crossPair[6 * i + n];
				matchedpFin2[6 * (matchPairs->numbasepair - 1) + n] = pair2[6 * choosenPair + n];
			}
		}else if(choosenPair == -1){
			/* if the crossing pair is not matched*/
			matchPairs->score -= rweight3() * rPENMissMotif();
		}
	}
	free((void *) seq_b1);
	free((void *) seq_b2);
	free((void *) matchedTargetIndex);
	return (matchPairs->score);
}

double ComputeIntervalScore(int startMotif, int endMotif, int startTarget, int endTarget, char *seq1, int seqLeng1, char *seq2, int seqLeng2, 
				int *unmatchedPairs1, int numUnmatched1, int *unmatchedPairs2, int numUnmatched2, int missPermission, int preloc){
	extern int W[15][15];
	int i, j, k, l;
	double score;
	int tempSeqLeng1, tempSeqLeng2;
	char *tempSeq1, *tempSeq2;
	int *AlignOps;
	
	//printf("segment 7\n");
	AlignOps = (int *) calloc((seqLeng1 + seqLeng2 + 1), sizeof(int));
	//printf("Seq lengths:	%d	%d\n", seqLeng1, seqLeng2);
	tempSeq1 = (char *) calloc(seqLeng1, sizeof(char));
	//printf("after allocation of tempSeq1\n");
	tempSeq2 = (char *) calloc(seqLeng2, sizeof(char));
	//printf("after allocation of tempSeq2\n");
	
	
	//printf("segment 6\n");
	
	/* gets the sequence need to be aligned*/
	tempSeqLeng1 = 0;
	tempSeqLeng2 = 0;
	for(l = startMotif+1; l < endMotif; ++ l){
		tempSeq1[tempSeqLeng1] = Char2int(seq1[l]);
		++tempSeqLeng1;
	}
	for(l = startTarget+1; l < endTarget; ++ l){
		tempSeq2[tempSeqLeng2] = Char2int(seq2[l]);
		++tempSeqLeng2;
	}
	/*
	printf("segment 4\n");
	*/
	score = rweight1() * (double) ALIGN(&tempSeq1[-1], &tempSeq2[-1], tempSeqLeng1, tempSeqLeng2, W, rGAPST(), rGAPET(), AlignOps);
	/*
	printf("segment 5\n");
	*/
	/* dealing with the motif sequence*/
	for(i = 0; i < numUnmatched1; ++ i){
		if(unmatchedPairs1[6 * i] >= startMotif && unmatchedPairs1[6 * i + 3] <= endMotif){
			/* substract the penalty and mark the base-pair as already penalized*/
			score -= rweight3() * rPENMissMotif();
			unmatchedPairs1[6 * i] = -1;
			unmatchedPairs1[6 * i + 3] = -1; 
		}
	} 
	/* dealing with the target sequence*/
	for(i = 0; i < numUnmatched2; ++ i){
		if(unmatchedPairs2[6 * i] >= startTarget && unmatchedPairs2[6 * i + 3] <= endTarget){
			/* substract the penalty and mark the base-pair as already penalized*/
			score -= rweight3() * rPENMissTarget();
			unmatchedPairs2[6 * i] = -1;
			unmatchedPairs2[6 * i + 3] = -1; 
		}
	}
	if(score < 0 && missPermission == 1){
		score = 0.0;
	}
	free((void*) tempSeq1);
	free((void*) tempSeq2);
	free((void*) AlignOps);
	return score;
}

double CheckScore(int *pair1, int numPairs1, int *pair2, int numPairs2, int *crossPair, int numCross, 
			int *matchedpFin1, int *matchedpFin2, int Prematched, int nummatched, 
			char *seq1, int seqLeng1, char *seq2, int seqLeng2, int numCleavage, int *Cleavages){
	extern int W[15][15];
	int i, j, k, l;
	double score = 0;
	int *intervalMotif, *intervalTarget;
	char *seq_b1 = (char *) calloc (seqLeng1, sizeof(char));
	char *seq_b2 = (char *) calloc (seqLeng2, sizeof(char));
	//printf("numPairs1:	%d	numPairs2:	%d\n", numPairs1, numPairs2);
	int *unmatchedPairs1 = (int *) calloc (6 * numPairs1, sizeof(int));
	int *unmatchedPairs2 = (int *) calloc (6 * numPairs2, sizeof(int));
	int numUnmatched1 = 0, numUnmatched2 = 0, numUnmatchedCross = 0;
	int numSegment = 0;
	int index1Motif = 0, index2Motif = 0, index1Target = 0, index2Target = 0;
	/* change the sequence into numerical representation*/
	for(i=0; i<seqLeng1; ++ i){
		seq_b1[i] = Char2int(seq1[i]);
	}
	for(i=0; i<seqLeng2; ++ i){
		seq_b2[i] = Char2int(seq2[i]);
	}
	/* find out how many base-pairs are un-matched*/
	int matchTag = 0;
	/* for the motif sequence*/
	for(i = 0; i < numPairs1; ++ i){
		matchTag = 0;
		/* find it in the list of matched base-pairs*/
		for(j = 0; j < Prematched; ++j){
			if(pair1[6 * i] == matchedpFin1[6 * j] && pair1[6 * i + 1] == matchedpFin1[6 * j + 1] 
				&& pair1[6 * i + 2] == matchedpFin1[6 * j + 2] && pair1[6 * i + 3] == matchedpFin1[6 * j + 3]
				&& pair1[6 * i + 4] == matchedpFin1[6 * j + 4] && pair1[6 * i + 5] == matchedpFin1[6 * j + 5]){
				matchTag = 1;
				break;
			}
		}
		/* if the base-pair is not matched*/
		if(matchTag == 0){
			/* copy the base-pair to unpair region*/
			for(j = 0; j < 6; ++j){
				unmatchedPairs1[6 * numUnmatched1 + j] = pair1[6 * i + j];
			}
			/* increase the number of unmatched base-pairs*/
			++ numUnmatched1;
		}
	}
	/* for the target sequence*/
	for(i = 0; i < numPairs2; ++ i){
		matchTag = 0;
		/* find it in the list of matched base-pairs*/
		for(j = 0; j < Prematched; ++j){
			if(pair2[6 * i] == matchedpFin2[6 * j] && pair2[6 * i + 1] == matchedpFin2[6 * j + 1] 
				&& pair2[6 * i + 2] == matchedpFin2[6 * j + 2] && pair2[6 * i + 3] == matchedpFin2[6 * j + 3]
				&& pair2[6 * i + 4] == matchedpFin2[6 * j + 4] && pair2[6 * i + 5] == matchedpFin2[6 * j + 5]){
				matchTag = 1;
				break;
			}
		}
		/* if the base-pair is not matched*/
		if(matchTag == 0){
			/* copy the base-pair to unpair region*/
			for(j = 0; j < 6; ++j){
				unmatchedPairs2[6 * numUnmatched2 + j] = pair2[6 * i + j];
			}
			/* increase the number of unmatched base-pairs*/
			++ numUnmatched2;
		}
	}
	/*
	printf("num unmatched1: %d	num unmatched2: %d\n", numUnmatched1, numUnmatched2);
	printf("printing unmatched pair in motif\n");
	for(i=0; i<numUnmatched1; ++i){
		for(j=0; j<6; ++j){
			printf("%d	", unmatchedPairs1[6 * i + j]);
		}
		printf("\n");
	}
	printf("printing unmatched pair in target\n");
	for(i=0; i<numUnmatched2; ++i){
		for(j=0; j<6; ++j){
			printf("%d	", unmatchedPairs2[6 * i + j]);
		}
		printf("\n");
	}
	*/
	/* checking for the matched base-pair score*/
	int contStacking = 0;
	double stackingBonus;
	double groupBonus;
	//printf("seqLeng1:	%d	seqLeng2:	%d\n", seqLeng1, seqLeng2);
	int *occupiedSeq1 = (int *) calloc (seqLeng1, sizeof(int));
	int *occupiedSeq2 = (int *) calloc (seqLeng2, sizeof(int));
	for(i = 0; i < seqLeng1; ++ i){
		occupiedSeq1[i] = 0;
	}
	for(i = 0; i < seqLeng2; ++ i){
		occupiedSeq2[i] = 0;
	}
	/* for each matched base-pair*/
	for(i = 0; i < Prematched; ++ i){
		/* add up the structural similarity score*/
		score += rweight3() * RetrievePairScore(seq_b1, seq_b2, &matchedpFin1[6 * i], &matchedpFin2[6 *i]);
		/*
		printf("score added, after checking structural score: %f\n", score);
		*/
		/* add up the sequence similarity score within the paired region*/
		score += rweight2() * (double) 
			(W[seq_b1[matchedpFin1[6 * i]]][seq_b2[matchedpFin2[6 * i]]] + W[seq_b1[matchedpFin1[6 * i + 3]]][seq_b2[matchedpFin2[6 * i + 3]]]);
		/*
		printf("score added, after checking paired region sequence score: %f\n", score);
		*/
		/* check for stacking effect bonus*/		
		if(i > 0 && matchedpFin1[6 * i] == matchedpFin1[6 * (i-1)] + 1 && matchedpFin2[6 * i] == matchedpFin2[6 * (i-1)] + 1 &&
			matchedpFin1[6 * i + 3] == matchedpFin1[6 * (i-1) + 3] - 1 && matchedpFin2[6 * i + 3] == matchedpFin2[6 * (i-1) + 3] - 1){
			++ contStacking; 
		}else{
			contStacking = 0;
		}
		/*
		printf("contStacking: %d\n", contStacking);
		*/
		score += contStacking * rweight3() * rBonusStacking();
		/*
		printf("score added: after checking stacking effect: %f\n", score);
		*/
		/* check for same group bonus*/
		/* if no conflict*/
		if(occupiedSeq1[matchedpFin1[6 * i]] == 0 && occupiedSeq1[matchedpFin1[6 * i + 3]] == 0){
			++ occupiedSeq1[matchedpFin1[6 * i]];
			++ occupiedSeq1[matchedpFin1[6 * i + 3]];
			++ occupiedSeq2[matchedpFin2[6 * i]];
			++ occupiedSeq2[matchedpFin2[6 * i + 3]];
		}else{
			/* if there are 1 - n pairings*/
			for(j = 0; j < i; ++j){
				if(matchedpFin1[6 * j] == matchedpFin1[6 * i] && matchedpFin1[6 * j + 3] > matchedpFin1[6 * i + 3]){
					/* add the bonus score and substract the sequence similarity becuase it is calculated twice*/
					/*
					printf("added SameG\n");
					*/
					score += rweight3() * rBonusSameG();
					score -= rweight2() * W[seq_b1[matchedpFin1[6 * i]]][seq_b2[matchedpFin2[6 * i]]];	
				}else if(matchedpFin1[6 * j + 3] == matchedpFin1[6 * i + 3] && matchedpFin1[6 * j] < matchedpFin1[6 * i]){
					score += rweight3() * rBonusSameG();
					score -= rweight2() * W[seq_b1[matchedpFin1[6 * i + 3]]][seq_b2[matchedpFin2[6 * i + 3]]];
				}else if(matchedpFin1[6 * j + 3] == matchedpFin1[6 * i]){
					score += rweight3() * rBonusSameH();
					score -= rweight2() * W[seq_b1[matchedpFin1[6 * i]]][seq_b2[matchedpFin2[6 * i]]];
				}
			}
			++ occupiedSeq1[matchedpFin1[6 * i]];
			++ occupiedSeq1[matchedpFin1[6 * i + 3]];
			++ occupiedSeq2[matchedpFin2[6 * i]];
			++ occupiedSeq2[matchedpFin2[6 * i + 3]];
		}
		/*
		printf("score added: after checking 1-n pairing: %f\n", score);
		*/
	}
	/*
	printf("Prematched: %d\n", Prematched);
	printf("score after checking base-pairs: %f\n", score);
	*/
	/* check how many segments*/
	for(i = 0; i < seqLeng1; ++ i){
		if(occupiedSeq1[i] != 0){
			++numSegment;
		}
	}
	/*
	for(i = 0; i < seqLeng1; ++ i){
		printf("%d	", occupiedSeq1[i]);
	}
	printf("\n");
	for(i = 0; i < seqLeng2; ++ i){
		printf("%d	", occupiedSeq2[i]);
	}
	printf("\n");
	*/
	/*
	printf("segment 1\n");
	*/
	/* put the segments' indices in to another array*/
	//printf("numSegment:	%d\n", numSegment);
	intervalMotif = (int *) calloc(numSegment + 1, sizeof(int));
	//printf("after the first allocation\n");
	intervalTarget = (int *) calloc(numSegment + 1, sizeof(int));
	//printf("after the second after allocation\n");
	j=0;
	for(i=0; i<seqLeng1; ++i){
		if(occupiedSeq1[i] > 0){
			intervalMotif[j] = i;
			++j;
		}
	}
	j=0;
	for(i=0; i<seqLeng2; ++i){
		if(occupiedSeq2[i] > 0){
			intervalTarget[j] = i;
			++j;
		}
	}
	//printf("after assigning intervals\n");
	/*
	printf("segment 2\n");
	*/
	/* check the starting of the intervals*/
	if(intervalMotif[0] > 0 || intervalTarget[0] > 0){
		//printf("before calling ComputeIntervalScore\n");
		score += ComputeIntervalScore(-1, intervalMotif[0], -1, intervalTarget[0], seq1, seqLeng1, seq2, seqLeng2,
				unmatchedPairs1, numUnmatched1, unmatchedPairs2, numUnmatched2, 1, 1);
		//printf("after calling ComputeIntervalScore\n");
	}
	//printf("after checking the first interval\n");
	/*
	printf("segment 3\n");
	*/
	/* for each inner segment*/
	for(i = 0; i < numSegment - 1; ++ i){
		index1Motif = intervalMotif[i];
		index2Motif = intervalMotif[i+1];
		index1Target = intervalTarget[i];
		index2Target = intervalTarget[i+1];
		/* if there is no sequence inside this interval*/
		if(index2Motif - index1Motif > 1 || index2Target - index1Target > 1){
			/*
			printf("checking segments\n");
			printf("%d	%d	%d	%d\n", index1Motif, index2Motif, index1Target, index2Target);
			*/
			if(numCleavage == 0){
				score += ComputeIntervalScore(index1Motif, index2Motif, index1Target, index2Target, seq1, seqLeng1, seq2, seqLeng2,
						unmatchedPairs1, numUnmatched1, unmatchedPairs2, numUnmatched2, 0, 0);
						/*
						printf("without permission: %f\n", score);
						*/
			}else{
				for(k = 0; k < numCleavage; ++ k){
					if(index1Motif < Cleavages[k] && index2Motif >= Cleavages[k]){
						score += ComputeIntervalScore(index1Motif, index2Motif, index1Target, index2Target, 
						seq1, seqLeng1, seq2, seqLeng2,	unmatchedPairs1, numUnmatched1, unmatchedPairs2, numUnmatched2, 1, 0);
						/*
						printf("with permission: %f\n", score);
						*/
						break;
					}else{
						score += ComputeIntervalScore(index1Motif, index2Motif, index1Target, index2Target, 
						seq1, seqLeng1, seq2, seqLeng2, unmatchedPairs1, numUnmatched1, unmatchedPairs2, numUnmatched2, 0, 0);
						/*
						printf("without permission: %f\n", score);
						*/
						break;
					}
				}
			}
		}
	}
	//printf("after checking the middle intervals\n");
	/* check the last interval*/
	if(index2Motif+1 < seqLeng1 || index2Target+1 < seqLeng2){
		score += ComputeIntervalScore(index2Motif, seqLeng1, index2Target, seqLeng2, seq1, seqLeng1, seq2, seqLeng2,
				unmatchedPairs1, numUnmatched1, unmatchedPairs2, numUnmatched2, 1, 0);
	}
	//printf("after checking pairs\n");
	/* check how many crossing base-pairs are unmatched*/
	for(i = 0; i < numCross; ++ i){
		matchTag = 0;
		for(j = Prematched; j < nummatched; ++ j){
			if(crossPair[6 * i] == matchedpFin1[6 * j] && crossPair[6 * i + 1] == matchedpFin1[6 * j + 1] &&
				crossPair[6 * i + 2] == matchedpFin1[6 * j + 2] && crossPair[6 * i + 3] == matchedpFin1[6 * j + 3] &&
				crossPair[6 * i + 4] == matchedpFin1[6 * j + 4] && crossPair[6 * i + 5] == matchedpFin1[6 * j + 5]){
				matchTag = 1;
				break;
			}
		}
		/* if the crossing pair is not matched*/
		if(matchTag == 0){
			++numUnmatchedCross;
		}
	}
	/*
	printf("score after checking intervals: %f\n", score);
	*/
	/* check for after checked pairing*/
	for(i = Prematched; i < nummatched; ++ i){
		/* check for the pair score*/
		score += rweight3() * RetrievePairScore(seq_b1, seq_b2, &matchedpFin1[6 * i], &matchedpFin2[6 *i]);
		/* add the sequence score*/
		if(occupiedSeq1[matchedpFin1[6 * i]] == 0){
			/*
			printf("rweight2: %f\n", rweight2());
			printf("weight sequence: %f\n", (double)  W[seq_b1[matchedpFin1[6 * i]]][seq_b2[matchedpFin2[6 * i]]]);
			printf("added score: %f\n", rweight2() * (double) W[seq_b1[matchedpFin1[6 * i]]][seq_b2[matchedpFin2[6 * i]]]);
			*/
			score = score + (rweight2() * (double) W[seq_b1[matchedpFin1[6 * i]]][seq_b2[matchedpFin2[6 * i]]]);
			/*
			printf("score added sequence1: %f\n", score);
			*/ 
		}
		if(occupiedSeq1[matchedpFin1[6 * i + 3]] == 0){
			/*
			printf("rweight2 %f\n", rweight2());
			printf("weight sequence: %f\n", (double)  W[seq_b1[matchedpFin1[6 * i + 3]]][seq_b2[matchedpFin2[6 * i + 3]]]);
			printf("added score: %f\n", rweight2() * (double) W[seq_b1[matchedpFin1[6 * i + 3]]][seq_b2[matchedpFin2[6 * i + 3]]]);
			*/
			score = score + (rweight2() * (double) W[seq_b1[matchedpFin1[6 * i + 3]]][seq_b2[matchedpFin2[6 * i + 3]]]);
			/*
			printf("score added sequence2: %f\n", score);
			*/
		}
		/* add the same group bonus*/
		if(occupiedSeq1[matchedpFin1[6 * i]] != 0){
			score += rweight3() * (rBonusSameG() + rBonusSameH())/2;
			/*
			printf("score added1: %f\n", score);
			*/
		}
		if(occupiedSeq1[matchedpFin1[6 * i + 3]] != 0){
			score += rweight3() * (rBonusSameG() + rBonusSameH())/2;
			/*
			printf("score added2: %f\n", score);
			*/
		}
		++ occupiedSeq1[matchedpFin1[6 * i]];
		++ occupiedSeq1[matchedpFin1[6 * i + 3]];
	}
	/*
	printf("after checking after check: %f\n", score);
	*/
	/* substract the unmatched pair's missing penalty*/
	for(i = 0; i < numUnmatched1; ++ i){
		if(unmatchedPairs1[6 * i] >= 0 && unmatchedPairs1[6 * i + 3] >= 0 && 
			unmatchedPairs1[6 * i] >= intervalMotif[0] && unmatchedPairs1[6 * i + 3] <= index2Motif){
			score -= rweight3() * rPENMissMotif();
		}
	}
	for(i = 0; i < numUnmatched2; ++ i){
		if(unmatchedPairs2[6 * i] >= 0 && unmatchedPairs2[6 * i + 3] >= 0 &&
			unmatchedPairs2[6 * i] >= intervalTarget[0] && unmatchedPairs2[6 * i + 3] <= index2Target){
			score -= rweight3() * rPENMissTarget();
		}
	}
	score -= numUnmatchedCross * rweight3() * rPENMissMotif();
	score += (nummatched - Prematched) * rweight3() * rPENMissTarget();
	//printf("before free\n");
	/*
	printf("nummatched: %d	Prematched: %d\n", nummatched, Prematched);
	printf("the score is %f\n", score);
	*/
	free((void *) intervalMotif);
	free((void *) intervalTarget);
	free((void *) occupiedSeq1);
	free((void *) occupiedSeq2);
	free((void *) unmatchedPairs1);
	free((void *) unmatchedPairs2);
	free((void *) seq_b1);
	free((void *) seq_b2);
	return score;
}
