#include <stdio.h>
#include <stdlib.h>
#include "./inc/RetrieveScore.h"
#include "./inc/ReadInput.h"


/*
 * HashIso
 *
 * The function takes an encoded nucleotide as input
 * and returns its numerical value for indexing in isosteric matrix
 * It convers the order G A C U into A C G U
*/
int HashIso(char n){
	switch(n){
		case 0:
			return 2;
		case 1:
			return 0;
		case 2:
			return 1;
		case 3:
			return 3;
		default:
			return -1;
	}
}

/*
 * RetrieveIsoGroup
 *
 * The function takes interacting edges and relative base orientation
 * and returns the isosteric class index that the base-pair belongs to
 * The class index is defined in Dr. Westhof's paper: NB Leontis, J Stombaugh, and E Westhof, NAR2002
 * However, the total isosteric groups are extend to 18, indexing of base-pairs are W:1 H:2 S:3
*/
int RetrieveIsoGroup(int edge1, int edge2, int orientation){
	if(edge1 >= 1 && edge1 <=3 && edge2 >=1 && edge2 <=3){
		/* convert from numerical representation to index*/	
		--edge1;
		--edge2;
		return ( (edge1 * 3 + edge2) * 2 + orientation);
	}else{
		printf("Error in interpreting edge information!\n");
		exit(0);
	}
}

/*
 * RetrievePairScore
 *
 * The arguement list contains: the first sequence, the second sequence, the index of pair in the first sequence, the index of pair in the second sequence
 * the isosteric matrix index, and the scoring matrix.
 * The return value is the Retrieved score for matching the two base-pairs if succesfully found the pair
 * If the pair is theoretically unfavarable, or there are other indexing errors, the function will terminate the program execution 
 * The function calls another two functions: RetrieveIsoGroup and HashIso
*/
double RetrievePairScore(char *sequence1, char *sequence2, int *pair1, int *pair2){
	extern int isoIndex[18][4][4];
	extern double scoreMatrix[66][66];
	// check for consistency of base-pair orientation
	if(pair1[5]!=pair1[2] || pair2[5]!=pair2[2]){
		printf("Error! Orientation of pair do no match!\n");
		exit(0);
	}
	// the index of the base-pair, extracted from isoIndex array
	int pairIndex1;
	int pairIndex2;
	// used to store the return values of the indexing functions
	int isoGroup;
	int isoNucleotide1;
	int isoNucleotide2;

	// processing the first pair
	isoGroup = RetrieveIsoGroup(pair1[1], pair1[4], pair1[2]);
	if(isoGroup < 0){	// may return some base-pairs that do not exist theoretically
		return 4.0;
		//	printf("Error! Please check for motif or target input\n");
		//	exit(0);
	}
	isoNucleotide1 = HashIso(sequence1[pair1[0]]);
	isoNucleotide2 = HashIso(sequence1[pair1[3]]);
	pairIndex1 = QueryIsoIndex(isoGroup, isoNucleotide1, isoNucleotide2);
	if(pairIndex1 == -1){
		return 4.0;
		//	printf("In motif structure: %d - %d\n", pair1[0], pair1[3]);
		//	printf("The structure exists some basepair that cannot exist theoretically!\n");
		//	printf("Please check the input files, program terminated.\n");
		//	exit(0);
	}
	// processing the second pair
	isoGroup = RetrieveIsoGroup(pair2[1], pair2[4], pair2[2]);						
	if(isoGroup < 0){	// may return some base-pairs that do not exist theoretically 
		return 4.0;
		//	printf("Error! Please check for motif or target input\n");
		//	exit(0);
	}
	isoNucleotide1 = HashIso(sequence2[pair2[0]]);
	isoNucleotide2 = HashIso(sequence2[pair2[3]]);
	pairIndex2 = QueryIsoIndex(isoGroup, isoNucleotide1, isoNucleotide2);
	if(pairIndex2 == -1){
		return 4.0;
		//	printf("In target structure: %d - %d\n", pair2[0], pair2[3]);
		//	printf("The structure exists some basepair that cannot exist theoretically!\n");
		//	printf("Please check the input files, program terminated.\n");
		//	exit(0);
	}
	// look up the scoring matrix
	if(pairIndex1 <= 66 && pairIndex1 > 0 && pairIndex2 <= 66 && pairIndex2 > 0){
		/* be careful that the indexing of isosteric group is staring from 1, to 66*/
		return QueryScore(pairIndex1-1, pairIndex2-1);
	}else{
		printf("Error! Wrong Index returned regarding base-pair score.\n");
		exit(0);
	}
	return;
}
int IsValidPair(int *pair, char *sequence){
	int pairIndex;
	int isoGroup;
	int isoNucleotide1;
	int isoNucleotide2;
	isoGroup = RetrieveIsoGroup(pair[1], pair[4], pair[2]);
	if(isoGroup < 0){	// may return some base-pairs that do not exist theoretically
		printf("Error! Please check for motif or target input\n");
		exit(0);
	}
	isoNucleotide1 = HashIso(sequence[pair[0]]);
	isoNucleotide2 = HashIso(sequence[pair[3]]);
	//printf("isoGroup: %d	isoNucleotide1: %d	isoNucleotide2: %d\n", isoGroup, isoNucleotide1, isoNucleotide2);
	pairIndex = QueryIsoIndex(isoGroup, isoNucleotide1, isoNucleotide2);
	//printf("retrieved group: %d\n", pairIndex);
	if(pairIndex == -1){
		return 0;
	}else{
		return 1;
	}
}

int IsSameIsoGroup(char *sequence1, char *sequence2, int *pair1, int *pair2){
	extern int isoIndex[18][4][4];
	extern double scoreMatrix[66][66];
	// check for consistency of base-pair orientation
	if(pair1[5]!=pair1[2] || pair2[5]!=pair2[2]){
		printf("Error! Orientation of pair do no match!\n");
		exit(0);
	}
	// the index of the base-pair, extracted from isoIndex array
	int pairIndex1;
	int pairIndex2;
	// used to store the return values of the indexing functions
	int isoGroup;
	int isoNucleotide1;
	int isoNucleotide2;

	// processing the first pair
	isoGroup = RetrieveIsoGroup(pair1[1], pair1[4], pair1[2]);
	if(isoGroup < 0){	// may return some base-pairs that do not exist theoretically
		printf("Error! Please check for motif or target input\n");
		exit(0);
	}
	isoNucleotide1 = HashIso(sequence1[pair1[0]]);
	isoNucleotide2 = HashIso(sequence1[pair1[3]]);
	pairIndex1 = QueryIsoIndex(isoGroup, isoNucleotide1, isoNucleotide2);
	if(pairIndex1 == -1){
		return 0;
		//	printf("In motif structure: %d - %d\n", pair1[0], pair1[3]);
		//	printf("The structure exists some basepair that cannot exist theoretically!\n");
		//	printf("Please check the input files, program terminated.\n");
		//	exit(0);
	}
	// processing the second pair
	isoGroup = RetrieveIsoGroup(pair2[1], pair2[4], pair2[2]);						
	if(isoGroup < 0){	// may return some base-pairs that do not exist theoretically 
		printf("Error! Please check for motif or target input\n");
		exit(0);
	}
	isoNucleotide1 = HashIso(sequence2[pair2[0]]);
	isoNucleotide2 = HashIso(sequence2[pair2[3]]);
	pairIndex2 = QueryIsoIndex(isoGroup, isoNucleotide1, isoNucleotide2);
	if(pairIndex2 == -1){
		return 0;
		//	printf("In target structure: %d - %d\n", pair2[0], pair2[3]);
		//	printf("The structure exists some basepair that cannot exist theoretically!\n");
		//	printf("Please check the input files, program terminated.\n");
		//	exit(0);
	}
	// look up the scoring matrix
	if(pairIndex1 <= 66 && pairIndex1 > 0 && pairIndex2 <= 66 && pairIndex2 > 0){
		if(pairIndex1 == pairIndex2){
			return 1;
		}else{
			return 0;
		}
	}else{
		printf("Error! Wrong Index returned regarding base-pair score.\n");
		exit(0);
	}
	return;
}

